#--
# NilComparable
#
# Copyright (c) 2005 Thomas Sawyer
#
# Ruby License
#
# This module is free software. You may use, modify, and/or redistribute this
# software under the same terms as Ruby.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.
#
#
# ==========================================================================
# Revision History ::
# --------------------------------------------------------------------------
#  5.4   trans     * Documentation.
# ==========================================================================
#++

#:title: NilComparable
#
# NilComparable does two things. First it makes nil comparable, such that
# all things (except itself) are greater than it.
#
# Secondly it provides a module called NilComparable to include into any
# other class to allow it to compare itself to NilClass as the greater
# of the two.
#
# == Usage
#
#   TODO
#
# == Author(s)
#
# * Paul Brannan
# * Thomas Sawyer
#

module NilComparable

  def self.included( mod )
    mod.class_eval %{
      if method_defined?( :<=> )
        alias_method :compare_without_nil, :<=>
      else
        def compare_without_nil( other )
          raise TypeError, "Cannot compare \#{self.inspect} to \#{other.inspect}"
        end
      end
      def <=>( other )
        return 1 if other.nil?
        compare_without_nil( other )
      end
    }
  end

end


class NilClass
  include Comparable
  def <=>(x)
    x.nil? ? 0 : -1
  end
  alias_method( :cmp, :<=> )
end

# # Calling this method for a given class allows instances
# # of a class to be compared to nil using <=>.
# # Adapted from code by Paul Brannan (Ruby License)
# def nil_comparable(klass=self)
#   klass.class_eval %{
#     if method_defined?( :<=> )
#       alias_method :compare_without_nil, :<=>
#     else
#       def compare_without_nil( other )
#         raise TypeError, "Cannot compare \#{self.inspect} to \#{other.inspect}"
#       end
#     end
#     def <=>( other )
#       return 1 if other.nil?
#       compare_without_nil( other )
#     end
#   }
# end



#  _____         _
# |_   _|__  ___| |_
#   | |/ _ \/ __| __|
#   | |  __/\__ \ |_
#   |_|\___||___/\__|
#

=begin testing

  require 'test/unit'

  class Mock < String
    include NilComparable
  end

  class TC_NilClass_Comparable < Test::Unit::TestCase

    def test001
      assert_equal( 0, nil <=> nil )
      assert_equal( -1, nil <=> 4 )
      assert_equal( -1, nil <=> "a" )
      assert_equal( -1, nil <=> Object.new )
      assert_equal( 0, nil.cmp(nil) )
      assert( nil < 4 )
    end

    def test002
      m = Mock.new("A")
      assert_equal( 1, m <=> nil )
      assert_equal( -1, m <=> "B" )
      #assert_equal( 1, m.cmp(nil) )
      assert( m > nil )
    end

  end

=end
